###############################################################################
## Company      : 4JS
## Program      : pager.4gl
## Date         : 16/01/1996
## Version      : 1.01
##
## Title        : Pager from data file or data of standard input
##
## Update       : 19/10/98
##              : v1.01
##              : Add set_delimiter
###############################################################################

use channel

###############################################################################
##                                  MAIN
###############################################################################

MAIN
  DEFINE 
    curr_line CHAR(512),
    outname   CHAR(164),
    infile    CHAR(20),
    contread  SMALLINT,
    readline  SMALLINT,
    i,t,u,v   SMALLINT,
    tmp_str   CHAR(1024)

  DEFER INTERRUPT
  #################
  # Verifications #
  #################

  LET outname=""
  IF num_args() = 0 THEN
    LET infile = NULL
  ELSE
    LET outname = arg_val(1)
    IF outname = "help" or outname ="?" THEN
      CALL usage(0)
    ELSE
      IF outname = "-v" THEN
        DISPLAY "Pager of file or standard input Version 1.00.1a"
        EXIT PROGRAM 0
      ELSE
        IF outname = "-d" THEN
          IF num_args() <= 1 THEN
            DISPLAY "Bad number of parameters"
            CALL usage(1)
          END IF 
          LET outname=arg_val(2)
          WHENEVER ERROR CONTINUE
          DATABASE outname
          IF status < 0 THEN
            DISPLAY "Cannot access to database ",outname clipped
            EXIT PROGRAM 0
          END IF
          CALL fgl_report_use_temptable()
          WHENEVER ERROR STOP
          IF num_args() > 2 THEN
            LET outname=arg_val(3)
          END IF
        END IF
      END IF
    END IF
    LET infile="f1"
  END IF
  WHENEVER ERROR CONTINUE
  CALL channel::open_file(infile,outname,"r")
  IF status < 0 THEN
    display "fglpager: Cannot open file ",outname clipped
    EXIT PROGRAM 1
  END IF
  WHENEVER ERROR STOP
  CALL channel::set_delimiter(infile,"")

  CALL fgl_init4js()

  START REPORT pager 

  LET readline = FALSE
  CALL read_nextline(infile) RETURNING contread,curr_line
  WHILE contread
    LET readline = TRUE
    LET i = length(curr_line)
    LET v = 1
    FOR t = 1 TO i
      IF curr_line[t,t] = "\t" THEN
        LET u = t MOD 8
        CASE u
          WHEN 1
            LET tmp_str = tmp_str[1,v], "        "
            LET v = v + 8
          WHEN 2
            LET tmp_str = tmp_str[1,v], "       "
            LET v = v + 7
          WHEN 3
            LET tmp_str = tmp_str[1,v], "      "
            LET v = v + 6
          WHEN 4
            LET tmp_str = tmp_str[1,v], "     "
            LET v = v + 5
          WHEN 5
            LET tmp_str = tmp_str[1,v], "    "
            LET v = v + 4
          WHEN 6
            LET tmp_str = tmp_str[1,v], "   "
            LET v = v + 3
          WHEN 7
            LET tmp_str = tmp_str[1,v], "  "
            LET v = v + 2
          WHEN 0
            LET tmp_str = tmp_str[1,v], " "
            LET v = v + 1
        END CASE
      ELSE
        IF curr_line[t,t] != "
" THEN
          LET tmp_str = tmp_str[1,v], curr_line[t,t]
        END IF
        LET v = v + 1
      END IF
    END FOR
    IF i THEN
      LET curr_line = tmp_str clipped
    END IF
    OUTPUT TO REPORT pager(curr_line)
    IF int_flag THEN
      EXIT WHILE
    END IF
    CALL read_nextline(infile) RETURNING contread,curr_line
  END WHILE

  IF infile IS NOT NULL THEN
    CALL channel::close(infile)
  END IF

  CALL fgl_report_continue(TRUE)
  IF readline THEN
    FINISH REPORT pager
  END IF

END MAIN

###############################################################################
##                                  FUNCTION
###############################################################################

################################################################################
## FUNCTION usage : Usage of this program
## Parameters :
##  none
## Returnings :
##  none
################################################################################

FUNCTION usage(noexit)

   DEFINE noexit SMALLINT

  DISPLAY "Pager of file or standard input"
  DISPLAY "Usage: fglpager [-d dbname] [filename]"
  DISPLAY "   where -d dbname: with usage of temporary tables"
  EXIT PROGRAM noexit

END FUNCTION

################################################################################
## FUNCTION read_nextline : Read next line from file or standard input
## Parameters :
##  infile    : Channel descriptor or "" for standard input
## Returnings :
##  contread  : TRUE if something are reading
##  curr_line : Current line
################################################################################

FUNCTION read_nextline(infile)

  DEFINE infile    CHAR(20),
         contread  SMALLINT,
         curr_line CHAR(512)

  #IF infile IS NULL THEN
    #CALL stdin_nextline() RETURNING contread,curr_line
  #ELSE
    LET contread = channel::read(infile,[curr_line])
  #END IF

  RETURN contread,
         curr_line

END FUNCTION #read_nextline


################################################################################
## REPORT pager : Report that display data
## Parameters :
##  c : Current line
## Returnings :
##  none
################################################################################

REPORT pager(c)

  DEFINE c CHAR(255)

  OUTPUT
  # You can change length of page if you want
     PAGE LENGTH 66
  
     TOP MARGIN 0
     LEFT MARGIN 0
     BOTTOM MARGIN 0
# 4JS 
     RIGHT MARGIN 256
  
  FORMAT
  ON EVERY ROW
     PRINT c CLIPPED

END REPORT
